#define _CRT_SECURE_NO_WARNINGS
#define _CRT_NONSTDC_NO_DEPRECATE

#include <stdio.h>
#include <stdlib.h>
#define _USE_MATH_DEFINES
#include <math.h>

#include "MWISio.h"

double* readBinary(const char* filename, double* arr, int* length)
{
	int l;
	FILE* fp = fopen(filename,"rb");

	if(fp<=NULL) return 0;

	fseek( fp, 0, SEEK_END );

	l = ftell( fp );
	l/=sizeof(double);

	rewind( fp );

	if(arr<=NULL) arr = (double*)malloc(l*sizeof(double));
	fread(arr,sizeof(double),l,fp);

	fclose(fp);

	if(length>NULL)*length = l;

	return arr;
}

int saveBinary(const char* filename, double* arr, int length)
{
	FILE* fp = fopen(filename,"wb+");

	if(fp<=NULL) return 0;

	fwrite(arr,sizeof(double),length,fp);

	fclose(fp);

	return 1;
}

NODE_IDS* readNodeIds(const char* filename, NODE_IDS* ids, int* length)
{
	double* arr = NULL;
	int i,l = 0;
	arr = readBinary(filename, arr, &l);
	l/=2;
	if(ids<=NULL) ids = (NODE_IDS*)malloc(l*sizeof(NODE_IDS));
	for(i=0;i<l;i++)
	{
		ids[i].id1 = (int)arr[0*l+i]-1;
		ids[i].id2 = (int)arr[1*l+i]-1;
	}
	free(arr);

	if(length>NULL)*length = l;
	return ids;
}

double* readAdjMat(const char* filename, double* A, int* length)
{
	int l = 0;
	A = readBinary(filename, A, &l);
	l=(int)sqrt(l);

	if(length>NULL)*length = l;
	return A;
}

BOX* readBoxes(const char* filename, BOX* b, int* length)
{
	double* arr = NULL;
	int i,l = 0;
	arr = readBinary(filename, arr, &l);
	l/=5;
	if(b<=NULL) b = (BOX*)malloc(l*sizeof(BOX));
	for(i=0;i<l;i++)
	{
		b[i].x1 = (int)arr[0*l+i];
		b[i].y1 = (int)arr[1*l+i];
		b[i].x2 = (int)arr[2*l+i];
		b[i].y2 = (int)arr[3*l+i];
		b[i].confidence = (double)arr[4*l+i];
		b[i].id = -1;
	}
	free(arr);

	if(length>NULL)*length = l;
	return b;
}

int saveX(const char* filename, unsigned char* x, int length)
{
	int i;
	double* arr = (double*)malloc(length*sizeof(double));

	for(i=0;i<length;i++)
		arr[i] = x[i];

	i = saveBinary(filename, arr, length);
	free(arr);

	return i;
}

unsigned char* readX(const char* filename, unsigned char* x, int* length)
{
	double* arr = NULL;
	int i,l = 0;
	arr = readBinary(filename, arr, &l);

	if(x<=NULL) x = (unsigned char*)malloc(l*sizeof(unsigned char));
	for(i=0;i<l;i++)
	{
		x[i] = (unsigned char)arr[i];
	}
	free(arr);

	if(length>NULL)*length = l;
	return x;
}