#define _CRT_SECURE_NO_WARNINGS
#define _CRT_NONSTDC_NO_DEPRECATE

#include <stdio.h>
#include <stdlib.h>

#include <cv.h>
#include <highgui.h>

#include "utils.h"
#include "mainutils.h"
#include "MWIS.h"
#include "MWISio.h"

typedef
struct params
{
	IplImage* img,*detect,*track,*colSel;
	BOX* b;
	int n;
	int id;
	int nCols;
	unsigned char changedId,changedCol;
}
params;

void drawBox(IplImage* img,BOX* b, CvScalar color, int offset, int thickness)
{
	cvRectangle(img,cvPoint(b->x1-offset	,	b->y1-offset),
					cvPoint(b->x2+offset	,	b->y2+offset),
					color,thickness,8,0);
}

void drawBoxes(IplImage* img,BOX* b, int length, int thickness)
{
	int i;

	for(i=0;i<length;i++)
	{
		if(b[i].confidence>-1.5) 
			drawBox(img,b+i, cvScalar(0,255,0,0),0,thickness);
	}
}

void drawSelectedBoxes(IplImage* img,BOX* b, int length, int thickness, int maxId)
{
	int i;

	for(i=0;i<length;i++)
	{
		if(b[i].id>=0) 
			drawBox(img,b+i, getIdCol(b[i].id, maxId),0,thickness);
	}
}

void drawMathedBoxes(IplImage* img1,BOX* b1,
					 IplImage* img2,BOX* b2,
					 NODE_IDS* ids, int i)
{

	BOX*b;

	printf("ids: %d %d\n",ids[i].id1,ids[i].id2);

	b = b1 + ids[i].id1;
	cvRectangle(img1, cvPoint(b->x1	,	b->y1),
					  cvPoint(b->x2	,	b->y2),
					  cvScalar(0,255,0,0),1,8,0);

	b = b2 + ids[i].id2;
	cvRectangle(img2, cvPoint(b->x1	,	b->y1),
					  cvPoint(b->x2	,	b->y2),
					  cvScalar(0,255,0,0),1,8,0);
}

void cbSelRect( int event, int x, int y, int flags, void*param )
{
	int i,id;
	int xc,yc;
	double d,dmin;
	params* p = (params*)param;

    if( !p->img )
        return;

    if( p->img->origin )
        y = p->img->height-1 - y;

    if(x>60000)x=0;
	if(y>60000)y=0;
	x = min(max(x,0),p->img->width-1);
	y = min(max(y,0),p->img->height-1);

    switch( event )
    {
    case CV_EVENT_LBUTTONDOWN:
		id = p->id;
		p->id = -1;
		dmin = 1e20;
		//printf("down\n");
		for(i=0;i<p->n;i++)
		{
			if(p->b[i].confidence<=-1.5) continue;

			if(x>=p->b[i].x1 && x<=p->b[i].x2 && y>=p->b[i].y1 && y<=p->b[i].y2)
			{
				xc = (p->b[i].x1 + p->b[i].x2)/2;
				yc = (p->b[i].y1 + p->b[i].y2)/2;
				d = (x-xc)*(x-xc) + (y-yc)*(y-yc);
				if(dmin>d)
				{
					dmin = d;
					p->id = i;
				}
			}
		}
		p->changedId = (id!=p->id);
        break;
    }
}

void cbSelCol( int event, int x, int y, int flags, void*param )
{
	params* p = (params*)param;

    if( !p->img )
        return;

    if( p->img->origin )
        y = p->img->height-1 - y;

    /*if(x>60000)x=0;
	if(y>60000)y=0;
	x = min(max(x,0),p->img->width-1);
	y = min(max(y,0),p->img->height-1);*/

    switch( event )
    {
    case CV_EVENT_LBUTTONDOWN:
		if(p->id>=0)
		{
			p->b[p->id].id = (y*(p->nCols+1))/p->colSel->height;
			printf("%d %d\n",p->b[p->id].id,p->nCols);
			if(p->b[p->id].id >= p->nCols) 
				p->b[p->id].id = -1;
			p->changedCol = 1;
		}
        break;
    }
}

void drawColorWindow(IplImage* img, int nCols)
{
	int step = img->height/(nCols+1);
	int i;

	cvZero(img);
	for(i=0;i<nCols;i++)
	{
		cvRectangle(img,cvPoint(0	,	i*step),
					cvPoint(img->width	,	(i+1)*step),
					getIdCol(i,nCols),-1,8,0);
	}
}

void initColSel(params* p)
{
	p->nCols = 10;
	p->colSel = cvCreateImage(cvSize(100,p->nCols*50),IPL_DEPTH_8U,3);
	drawColorWindow(p->colSel, p->nCols);
}

/*
  *****************************************************************************
	Main function.

	@param argc number of arguments.
	@param argv array of arguments, the first argument being the name of the program.
	
	@return Returns 0 if went well.
*/
int main( int argc, char** argv )
{
	int frame = 330;
	const char* path = argv[1];
	const char* objectType = "Ped";
	char c;
	params param={0};
	param.id = -1;

	cvNamedWindow("frame", CV_WINDOW_AUTOSIZE);
	cvMoveWindow("frame", 1, 1);
	cvNamedWindow("select", CV_WINDOW_AUTOSIZE);
	cvMoveWindow("select", 1, 500);
	
	cvSetMouseCallback( "frame", cbSelRect, &param );

	initColSel(&param);
	cvNamedWindow("col", CV_WINDOW_AUTOSIZE);
	cvMoveWindow("col", 800, 1);
	cvShowImage("col",param.colSel);
	cvSetMouseCallback( "col", cbSelCol, &param );

	for(frame=208;frame<=365;frame++)
	{
		param.img = cvLoadImage(mkStr("%simage_00000%d_1.png",path,frame),1);
		param.detect = cvCloneImage(param.img);
		param.track = cvCloneImage(param.img);
		param.b = readBoxes(mkStr("%sBbox%s%d.bin",path,objectType,frame), NULL, &param.n);
		drawBoxes( param.detect, param.b, param.n,1);
		
		cvShowImage("frame",param.detect);
		cvShowImage("select",param.track);
		c = cvWaitKey(10);
		
		//c = param.b>NULL?0:' ';
		c = ' ';
		while(c!=' ')
		{
			c = 0;
			c = cvWaitKey(10);

			if(param.changedId)
			{
				param.changedId = 0;
				cvCopyImage(param.img,param.detect);
				cvCopyImage(param.img,param.track);
				drawBoxes( param.detect, param.b, param.n,1);
				drawSelectedBoxes( param.track, param.b, param.n,2,param.nCols);
				if(param.id>=0)
				{
					drawBox( param.detect, param.b + param.id, CV_RGB(255,0,255),1,2);
					drawBox( param.track, param.b + param.id, CV_RGB(255,0,255),1,2);
				}
				cvShowImage("frame",param.detect);
				cvShowImage("select",param.track);
			}

			if(param.changedCol)
			{
				param.changedCol = 0;
				cvCopyImage(param.img,param.track);
				drawSelectedBoxes( param.track, param.b, param.n,2,param.nCols);
				cvShowImage("select",param.track);
			}

			switch(c)
			{
			case 27:
				exit(0);
			/*default:
				printf("not a good key\n");*/
			}
		}

		cvCopyImage(param.img,param.detect);
		drawBoxes( param.detect, param.b, param.n,2);
		//cvSaveImage(mkStr("%sdetect_%d.png",path,frame),param.detect);
		/*cvCopyImage(param.img,param.track);
		drawSelectedBoxes( param.track, param.b, param.n,2,param.nCols);
		cvSaveImage(mkStr("%strack_%d.png",path,frame),param.track);*/
		cvSaveImage(mkStr("%sorig_%d.png",path,frame),param.img);

		free(param.b);
		cvReleaseImage(&param.img);
		cvReleaseImage(&param.detect);
		cvReleaseImage(&param.track);
		param.b = NULL;

		if(c==27)break;
	}

	return 0;
}

/*
  *****************************************************************************
	Main function.

	@param argc number of arguments.
	@param argv array of arguments, the first argument being the name of the program.
	
	@return Returns 0 if went well.
*/
int main2( int argc, char** argv )
{
	int frame1,frame2;
	int n,nb1,nb2;
	BOX* b1=NULL, *b2=NULL;
	double* weights = NULL;
	double* A = NULL;
	NODE_IDS* ids = NULL;
	unsigned char* x = NULL;
	const char* path;
	const char* objectType = "Ped";
	int i=0;

	IplImage* img1,*img2;

	if( argc < 4 )
	{
		printf("\nERROR: specify the frames!\n");
		printf("PRESS ENTER TO EXIT\n");
		getchar();
		return -3;
	}

	if(argc > 4)
		objectType = argv[4];

	path = argv[1];
	frame1 = atoi(argv[2]);
	frame2 = atoi(argv[3]);

	b1 = readBoxes(mkStr("%sBbox%s%d.bin",path,objectType,frame1), b1, &nb1);
	b2 = readBoxes(mkStr("%sBbox%s%d.bin",path,objectType,frame2), b2, &nb2);
	ids = readNodeIds(mkStr("%sNodes%s%d_%d.bin",path,objectType,frame1,frame2), ids, &n);
	A = readAdjMat(mkStr("%sAssociation%s%d_%d.bin",path,objectType,frame1,frame2), A, &n);
	weights = readWeights(mkStr("%sWeights%s%d_%d.bin",path,objectType,frame1,frame2), weights, &n);

	/* MWIS */
	x = mwisGetX( weights, A, n, 0);
	saveX(mkStr("%sx%s%d_%d.bin",path,objectType,frame1,frame2),  x, n);

	/* display images */
	img1 = cvLoadImage(mkStr("%simage_00000%d_1.png",path,frame1),1);
	img2 = cvLoadImage(mkStr("%simage_00000%d_1.png",path,frame2),1);

	cvNamedWindow("frame1", CV_WINDOW_AUTOSIZE);
	cvMoveWindow("frame1", 1, 300);
	cvNamedWindow("frame2", CV_WINDOW_AUTOSIZE);
	cvMoveWindow("frame2", 500, 300);/**/

	drawBoxes( img1, b1, nb1,1);
	drawBoxes( img2, b2, nb2,1);
	/*for(i=0;i<n;i++)
	{
		if(x[i])
		{
			//img1 = cvLoadImage(mkStr("%s%d.jpg",path,frame1),1);
			//img2 = cvLoadImage(mkStr("%s%d.jpg",path,frame2),1);
			drawMathedBoxes(img1, b1,
							img2, b2,
							ids , i);
			//cvShowImage("frame1",img1);
			//cvShowImage("frame2",img2);
			//cvWaitKey(0);
			//cvReleaseImage(&img1);
			//cvReleaseImage(&img2);
		}
	}*/

	cvShowImage("frame1",img1);
	cvShowImage("frame2",img2);

	cvWaitKey(0);

	return 0;
}